@@ -63,6 +63,9 @@
 #include <vis.h>
 #include <imsg.h>
 
+#ifndef HAVE_PLEDGE
+#include <openssl/x509.h>
+#endif
 #include <tls.h>
 
 #include "extern.h"
@@ -1809,6 +1812,15 @@ http_setup(void)
 		err(1, "tls_load_file: %s", tls_default_ca_cert_file());
 	tls_config_set_ca_mem(tls_config, tls_ca_mem, tls_ca_size);
 #endif
+#ifdef HAVE_LANDLOCK
+	/*
+	 * GROSS HACK: landlock has no concept of dns pledge and on top
+	 * of this libretls/OpenSSL don't support loading certs into memory.
+	 * Only solution is to give read access to the hole world.
+	 * Yes, this is a giant hole but landlock is just not fit for use.
+	 */
+	unveil("/", "r");
+#endif
 
 	if ((httpproxy = getenv("http_proxy")) != NULL && *httpproxy == '\0')
 		httpproxy = NULL;
