@@ -1,4 +1,4 @@
-/*	$OpenBSD: print.c,v 1.22 2022/12/01 12:41:34 job Exp $ */
+/*	$OpenBSD: print.c,v 1.34 2023/03/13 19:51:49 job Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -57,7 +57,8 @@ time2str(time_t t)
 	if (gmtime_r(&t, &tm) == NULL)
 		return "could not convert time";
 
-	strftime(buf, sizeof(buf), "%h %d %T %Y %Z", &tm);
+	strftime(buf, sizeof(buf), "%a %d %b %Y %T %z", &tm);
+
 	return buf;
 }
 
@@ -103,9 +104,12 @@ tal_print(const struct tal *p)
 	} else {
 		printf("Trust anchor name:        %s\n", p->descr);
 		printf("Subject key identifier:   %s\n", pretty_key_id(ski));
-		printf("Trust anchor locations:\n");
-		for (i = 0; i < p->urisz; i++)
-			printf("%5zu: %s\n", i + 1, p->uri[i]);
+		printf("Trust anchor locations:   ");
+		for (i = 0; i < p->urisz; i++) {
+			if (i > 0)
+				printf("%26s", "");
+			printf("%s\n", p->uri[i]);
+		}
 	}
 
 	EVP_PKEY_free(pk);
@@ -117,27 +121,38 @@ void
 x509_print(const X509 *x)
 {
 	const ASN1_INTEGER	*xserial;
+	const X509_NAME		*xissuer;
+	char			*issuer = NULL;
 	char			*serial = NULL;
 
-	xserial = X509_get0_serialNumber(x);
-	if (xserial == NULL) {
-		warnx("X509_get0_serialNumber failed in %s", __func__);
+	if ((xissuer = X509_get_issuer_name(x)) == NULL) {
+		warnx("X509_get_issuer_name failed");
 		goto out;
 	}
 
-	serial = x509_convert_seqnum(__func__, xserial);
-	if (serial == NULL) {
-		warnx("x509_convert_seqnum failed in %s", __func__);
+	if ((issuer = X509_NAME_oneline(xissuer, NULL, 0)) == NULL) {
+		warnx("X509_NAME_oneline failed");
 		goto out;
 	}
 
+	if ((xserial = X509_get0_serialNumber(x)) == NULL) {
+		warnx("X509_get0_serialNumber failed");
+		goto out;
+	}
+
+	if ((serial = x509_convert_seqnum(__func__, xserial)) == NULL)
+		goto out;
+
 	if (outformats & FORMAT_JSON) {
+		printf("\t\"cert_issuer\": \"%s\",\n", issuer);
 		printf("\t\"cert_serial\": \"%s\",\n", serial);
 	} else {
+		printf("Certificate issuer:       %s\n", issuer);
 		printf("Certificate serial:       %s\n", serial);
 	}
 
  out:
+	free(issuer);
 	free(serial);
 }
 
@@ -147,9 +162,6 @@ cert_print(const struct cert *p)
 	size_t			 i, j;
 	char			 buf1[64], buf2[64];
 	int			 sockt;
-	char			 tbuf[21];
-
-	strftime(tbuf, sizeof(tbuf), "%FT%TZ", gmtime(&p->expires));
 
 	if (outformats & FORMAT_JSON) {
 		if (p->pubkey != NULL)
@@ -170,7 +182,10 @@ cert_print(const struct cert *p)
 			printf("\t\"notify_url\": \"%s\",\n", p->notify);
 		if (p->pubkey != NULL)
 			printf("\t\"router_key\": \"%s\",\n", p->pubkey);
-		printf("\t\"valid_until\": %lld,\n", (long long)p->expires);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->notbefore);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->notafter);
+		if (p->expires)
+			printf("\t\"expires\": %lld,\n", (long long)p->expires);
 		printf("\t\"subordinate_resources\": [\n");
 	} else {
 		printf("Subject key identifier:   %s\n", pretty_key_id(p->ski));
@@ -189,10 +204,17 @@ cert_print(const struct cert *p)
 		if (p->pubkey != NULL) {
 			printf("BGPsec ECDSA public key:  %s\n",
 			    p->pubkey);
-			printf("Router key valid until:   %s\n", tbuf);
-		} else
-			printf("Certificate valid until:  %s\n", tbuf);
-		printf("Subordinate resources:\n");
+			printf("Router key not before:    %s\n",
+			    time2str(p->notbefore));
+			printf("Router key not after:     %s\n",
+			    time2str(p->notafter));
+		} else {
+			printf("Certificate not before:   %s\n",
+			    time2str(p->notbefore));
+			printf("Certificate not after:    %s\n",
+			    time2str(p->notafter));
+		}
+		printf("Subordinate resources:    ");
 	}
 
 	for (i = 0; i < p->asz; i++) {
@@ -200,23 +222,32 @@ cert_print(const struct cert *p)
 		case CERT_AS_ID:
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"asid\": %u }", p->as[i].id);
-			else
-				printf("%5zu: AS: %u", i + 1, p->as[i].id);
+			else {
+				if (i > 0)
+					printf("%26s", "");
+				printf("AS: %u", p->as[i].id);
+			}
 			break;
 		case CERT_AS_INHERIT:
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"asid_inherit\": \"true\" }");
-			else
-				printf("%5zu: AS: inherit", i + 1);
+			else {
+				if (i > 0)
+					printf("%26s", "");
+				printf("AS: inherit");
+			}
 			break;
 		case CERT_AS_RANGE:
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"asrange\": { \"min\": %u, "
 				    "\"max\": %u }}", p->as[i].range.min,
 				    p->as[i].range.max);
-			else
-				printf("%5zu: AS: %u -- %u", i + 1,
-				    p->as[i].range.min, p->as[i].range.max);
+			else {
+				if (i > 0)
+					printf("%26s", "");
+				printf("AS: %u -- %u", p->as[i].range.min,
+				    p->as[i].range.max);
+			}
 			break;
 		}
 		if (outformats & FORMAT_JSON && i + 1 < p->asz + p->ipsz)
@@ -230,16 +261,22 @@ cert_print(const struct cert *p)
 		case CERT_IP_INHERIT:
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"ip_inherit\": \"true\" }");
-			else
-				printf("%5zu: IP: inherit", i + j + 1);
+			else {
+				if (i > 0 || j > 0)
+					printf("%26s", "");
+				printf("IP: inherit");
+			}
 			break;
 		case CERT_IP_ADDR:
 			ip_addr_print(&p->ips[j].ip,
 			    p->ips[j].afi, buf1, sizeof(buf1));
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"ip_prefix\": \"%s\" }", buf1);
-			else
-				printf("%5zu: IP: %s", i + j + 1, buf1);
+			else {
+				if (i > 0 || j > 0)
+					printf("%26s", "");
+				printf("IP: %s", buf1);
+			}
 			break;
 		case CERT_IP_RANGE:
 			sockt = (p->ips[j].afi == AFI_IPV4) ?
@@ -249,9 +286,11 @@ cert_print(const struct cert *p)
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"ip_range\": { \"min\": \"%s\""
 				    ", \"max\": \"%s\" }}", buf1, buf2);
-			else
-				printf("%5zu: IP: %s -- %s", i + j + 1, buf1,
-				    buf2);
+			else {
+				if (i > 0 || j > 0)
+					printf("%26s", "");
+				printf("IP: %s -- %s", buf1, buf2);
+			}
 			break;
 		}
 		if (outformats & FORMAT_JSON && i + j + 1 < p->asz + p->ipsz)
@@ -270,8 +309,9 @@ crl_print(const struct crl *p)
 	STACK_OF(X509_REVOKED)	*revlist;
 	X509_REVOKED *rev;
 	ASN1_INTEGER *crlnum;
+	X509_NAME *xissuer;
 	int i;
-	char *serial;
+	char *issuer, *serial;
 	time_t t;
 
 	if (outformats & FORMAT_JSON) {
@@ -280,24 +320,32 @@ crl_print(const struct crl *p)
 	} else
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
 
+	xissuer = X509_CRL_get_issuer(p->x509_crl);
+	issuer = X509_NAME_oneline(xissuer, NULL, 0);
 	crlnum = X509_CRL_get_ext_d2i(p->x509_crl, NID_crl_number, NULL, NULL);
 	serial = x509_convert_seqnum(__func__, crlnum);
-	if (serial != NULL) {
-		if (outformats & FORMAT_JSON)
+	if (issuer != NULL && serial != NULL) {
+		if (outformats & FORMAT_JSON) {
+			printf("\t\"crl_issuer\": \"%s\",\n", issuer);
 			printf("\t\"crl_serial\": \"%s\",\n", serial);
-		else
-			printf("CRL Serial Number:        %s\n", serial);
+		} else {
+			printf("CRL issuer:               %s\n", issuer);
+			printf("CRL serial number:        %s\n", serial);
+		}
 	}
+	free(issuer);
 	free(serial);
 	ASN1_INTEGER_free(crlnum);
 
 	if (outformats & FORMAT_JSON) {
-		printf("\t\"valid_since\": %lld,\n", (long long)p->issued);
-		printf("\t\"valid_until\": %lld,\n", (long long)p->expires);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->lastupdate);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->nextupdate);
 		printf("\t\"revoked_certs\": [\n");
 	} else {
-		printf("CRL valid since:          %s\n", time2str(p->issued));
-		printf("CRL valid until:          %s\n", time2str(p->expires));
+		printf("CRL last update:          %s\n",
+		    time2str(p->lastupdate));
+		printf("CRL next update:          %s\n",
+		    time2str(p->nextupdate));
 		printf("Revoked Certificates:\n");
 	}
 
@@ -315,8 +363,8 @@ crl_print(const struct crl *p)
 					printf(",");
 				printf("\n");
 			} else
-				printf("    Serial: %8s   Revocation Date: %s"
-				    "\n", serial, time2str(t));
+				printf("%25s Serial: %8s   Revocation Date: %s"
+				    "\n", "", serial, time2str(t));
 		}
 		free(serial);
 	}
@@ -341,8 +389,13 @@ mft_print(const X509 *x, const struct mft *p)
 		printf("\t\"aia\": \"%s\",\n", p->aia);
 		printf("\t\"sia\": \"%s\",\n", p->sia);
 		printf("\t\"manifest_number\": \"%s\",\n", p->seqnum);
-		printf("\t\"valid_since\": %lld,\n", (long long)p->valid_since);
-		printf("\t\"valid_until\": %lld,\n", (long long)p->valid_until);
+		if (p->signtime != 0)
+			printf("\t\"signing_time\": %lld,\n",
+			    (long long)p->signtime);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->thisupdate);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->nextupdate);
+		if (p->expires)
+			printf("\t\"expires\": %lld,\n", (long long)p->expires);
 	} else {
 		printf("Subject key identifier:   %s\n", pretty_key_id(p->ski));
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
@@ -350,9 +403,12 @@ mft_print(const X509 *x, const struct mft *p)
 		printf("Authority info access:    %s\n", p->aia);
 		printf("Subject info access:      %s\n", p->sia);
 		printf("Manifest Number:          %s\n", p->seqnum);
-		printf("Manifest valid since:     %s\n", time2str(p->valid_since));
-		printf("Manifest valid until:     %s\n", time2str(p->valid_until));
-		printf("Files and hashes:\n");
+		if (p->signtime != 0)
+			printf("Signing time:             %s\n",
+			    time2str(p->signtime));
+		printf("Manifest this update:     %s\n", time2str(p->thisupdate));
+		printf("Manifest next update:     %s\n", time2str(p->nextupdate));
+		printf("Files and hashes:         ");
 	}
 
 	for (i = 0; i < p->filesz; i++) {
@@ -370,8 +426,10 @@ mft_print(const X509 *x, const struct mft *p)
 				printf(",");
 			printf("\n");
 		} else {
-			printf("%5zu: %s\n", i + 1, p->files[i].file);
-			printf("\thash %s\n", hash);
+			if (i > 0)
+				printf("%26s", "");
+			printf("%zu: %s (hash: %s)\n", i + 1, p->files[i].file,
+			    hash);
 		}
 
 		free(hash);
@@ -394,16 +452,27 @@ roa_print(const X509 *x, const struct roa *p)
 		printf("\t\"aki\": \"%s\",\n", pretty_key_id(p->aki));
 		printf("\t\"aia\": \"%s\",\n", p->aia);
 		printf("\t\"sia\": \"%s\",\n", p->sia);
-		printf("\t\"valid_until\": %lld,\n", (long long)p->expires);
+		if (p->signtime != 0)
+			printf("\t\"signing_time\": %lld,\n",
+			    (long long)p->signtime);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->notbefore);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->notafter);
+		if (p->expires)
+			printf("\t\"expires\": %lld,\n", (long long)p->expires);
 	} else {
 		printf("Subject key identifier:   %s\n", pretty_key_id(p->ski));
 		x509_print(x);
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
 		printf("Authority info access:    %s\n", p->aia);
 		printf("Subject info access:      %s\n", p->sia);
-		printf("ROA valid until:          %s\n", time2str(p->expires));
+		if (p->signtime != 0)
+			printf("Signing time:             %s\n",
+			    time2str(p->signtime));
+		printf("ROA not before:           %s\n",
+		    time2str(p->notbefore));
+		printf("ROA not after:            %s\n", time2str(p->notafter));
 		printf("asID:                     %u\n", p->asid);
-		printf("IP address blocks:\n");
+		printf("IP address blocks:        ");
 	}
 
 	for (i = 0; i < p->ipsz; i++) {
@@ -420,9 +489,11 @@ roa_print(const X509 *x, const struct roa *p)
 			if (i + 1 < p->ipsz)
 				printf(",");
 			printf("\n");
-		} else
-			printf("%5zu: %s maxlen: %hhu\n", i + 1, buf,
-			    p->ips[i].maxlength);
+		} else {
+			if (i > 0)
+				printf("%26s", "");
+			printf("%s maxlen: %hhu\n", buf, p->ips[i].maxlength);
+		}
 	}
 
 	if (outformats & FORMAT_JSON)
@@ -441,6 +512,13 @@ gbr_print(const X509 *x, const struct gbr *p)
 		printf("\t\"aki\": \"%s\",\n", pretty_key_id(p->aki));
 		printf("\t\"aia\": \"%s\",\n", p->aia);
 		printf("\t\"sia\": \"%s\",\n", p->sia);
+		if (p->signtime != 0)
+			printf("\t\"signing_time\": %lld,\n",
+			    (long long)p->signtime);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->notbefore);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->notafter);
+		if (p->expires)
+			printf("\t\"expires\": %lld,\n", (long long)p->expires);
 		printf("\t\"vcard\": \"");
 		for (i = 0; i < strlen(p->vcard); i++) {
 			if (p->vcard[i] == '"')
@@ -459,6 +537,12 @@ gbr_print(const X509 *x, const struct gbr *p)
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
 		printf("Authority info access:    %s\n", p->aia);
 		printf("Subject info access:      %s\n", p->sia);
+		if (p->signtime != 0)
+			printf("Signing time:             %s\n",
+			    time2str(p->signtime));
+		printf("GBR not before:           %s\n",
+		    time2str(p->notbefore));
+		printf("GBR not after:            %s\n", time2str(p->notafter));
 		printf("vcard:\n%s", p->vcard);
 	}
 }
@@ -466,27 +550,36 @@ gbr_print(const X509 *x, const struct gbr *p)
 void
 rsc_print(const X509 *x, const struct rsc *p)
 {
-	char	 buf1[64], buf2[64], tbuf[21];
+	char	 buf1[64], buf2[64];
 	char	*hash;
 	int	 sockt;
 	size_t	 i, j;
 
-	strftime(tbuf, sizeof(tbuf), "%FT%TZ", gmtime(&p->expires));
-
 	if (outformats & FORMAT_JSON) {
 		printf("\t\"ski\": \"%s\",\n", pretty_key_id(p->ski));
 		printf("\t\"aki\": \"%s\",\n", pretty_key_id(p->aki));
 		x509_print(x);
 		printf("\t\"aia\": \"%s\",\n", p->aia);
-		printf("\t\"valid_until\": %lld,\n", (long long)p->expires);
+		if (p->signtime != 0)
+			printf("\t\"signing_time\": %lld,\n",
+			    (long long)p->signtime);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->notbefore);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->notafter);
+		if (p->expires)
+			printf("\t\"expires\": %lld,\n", (long long)p->expires);
 		printf("\t\"signed_with_resources\": [\n");
 	} else {
 		printf("Subject key identifier:   %s\n", pretty_key_id(p->ski));
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
 		x509_print(x);
 		printf("Authority info access:    %s\n", p->aia);
-		printf("Valid until:              %s\n", tbuf);
-		printf("Signed with resources:\n");
+		if (p->signtime != 0)
+			printf("Signing time:             %s\n",
+			    time2str(p->signtime));
+		printf("RSC not before:           %s\n",
+		    time2str(p->notbefore));
+		printf("RSC not after:            %s\n", time2str(p->notafter));
+		printf("Signed with resources:    ");
 	}
 
 	for (i = 0; i < p->asz; i++) {
@@ -494,17 +587,23 @@ rsc_print(const X509 *x, const struct rsc *p)
 		case CERT_AS_ID:
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"asid\": %u }", p->as[i].id);
-			else
-				printf("%5zu: AS: %u", i + 1, p->as[i].id);
+			else {
+				if (i > 0)
+					printf("%26s", "");
+				printf("AS: %u", p->as[i].id);
+			}
 			break;
 		case CERT_AS_RANGE:
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"asrange\": { \"min\": %u, "
 				    "\"max\": %u }}", p->as[i].range.min,
 				    p->as[i].range.max);
-			else
-				printf("%5zu: AS: %u -- %u", i + 1,
-				    p->as[i].range.min, p->as[i].range.max);
+			else {
+				if (i > 0)
+					printf("%26s", "");
+				printf("AS: %u -- %u", p->as[i].range.min,
+				    p->as[i].range.max);
+			}
 			break;
 		case CERT_AS_INHERIT:
 			/* inheritance isn't possible in RSC */
@@ -523,8 +622,11 @@ rsc_print(const X509 *x, const struct rsc *p)
 			    p->ips[j].afi, buf1, sizeof(buf1));
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"ip_prefix\": \"%s\" }", buf1);
-			else
-				printf("%5zu: IP: %s", i + j + 1, buf1);
+			else {
+				if (i > 0 || j > 0)
+					printf("%26s", "");
+				printf("IP: %s", buf1);
+			}
 			break;
 		case CERT_IP_RANGE:
 			sockt = (p->ips[j].afi == AFI_IPV4) ?
@@ -534,9 +636,11 @@ rsc_print(const X509 *x, const struct rsc *p)
 			if (outformats & FORMAT_JSON)
 				printf("\t\t{ \"ip_range\": { \"min\": \"%s\""
 				    ", \"max\": \"%s\" }}", buf1, buf2);
-			else
-				printf("%5zu: IP: %s -- %s", i + j + 1, buf1,
-				    buf2);
+			else {
+				if (i > 0 || j > 0)
+					printf("%26s", "");
+				printf("IP: %s -- %s", buf1, buf2);
+			}
 			break;
 		case CERT_IP_INHERIT:
 			/* inheritance isn't possible in RSC */
@@ -552,7 +656,7 @@ rsc_print(const X509 *x, const struct rsc *p)
 		printf("\t],\n");
 		printf("\t\"filenamesandhashes\": [\n");
 	} else
-		printf("Filenames and hashes:\n");
+		printf("Filenames and hashes:     ");
 
 	for (i = 0; i < p->filesz; i++) {
 		if (base64_encode(p->files[i].hash, sizeof(p->files[i].hash),
@@ -567,9 +671,11 @@ rsc_print(const X509 *x, const struct rsc *p)
 				printf(",");
 			printf("\n");
 		} else {
-			printf("%5zu: %s\n", i + 1, p->files[i].filename
-			    ? p->files[i].filename : "no filename");
-			printf("\thash %s\n", hash);
+			if (i > 0)
+				printf("%26s", "");
+			printf("%zu: %s (hash: %s)\n", i + 1,
+			    p->files[i].filename ? p->files[i].filename
+			    : "no filename", hash);
 		}
 
 		free(hash);
@@ -591,6 +697,13 @@ aspa_print(const X509 *x, const struct aspa *p)
 		printf("\t\"aki\": \"%s\",\n", pretty_key_id(p->aki));
 		printf("\t\"aia\": \"%s\",\n", p->aia);
 		printf("\t\"sia\": \"%s\",\n", p->sia);
+		if (p->signtime != 0)
+			printf("\t\"signing_time\": %lld,\n",
+			    (long long)p->signtime);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->notbefore);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->notafter);
+		if (p->expires)
+			printf("\t\"expires\": %lld,\n", (long long)p->expires);
 		printf("\t\"customer_asid\": %u,\n", p->custasid);
 		printf("\t\"provider_set\": [\n");
 		for (i = 0; i < p->providersz; i++) {
@@ -611,10 +724,18 @@ aspa_print(const X509 *x, const struct aspa *p)
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
 		printf("Authority info access:    %s\n", p->aia);
 		printf("Subject info access:      %s\n", p->sia);
+		if (p->signtime != 0)
+			printf("Signing time:             %s\n",
+			    time2str(p->signtime));
+		printf("ASPA not before:          %s\n",
+		    time2str(p->notbefore));
+		printf("ASPA not after:           %s\n", time2str(p->notafter));
 		printf("Customer AS:              %u\n", p->custasid);
-		printf("Provider Set:\n");
+		printf("Provider Set:             ");
 		for (i = 0; i < p->providersz; i++) {
-			printf("%5zu: AS: %d", i + 1, p->providers[i].as);
+			if (i > 0)
+				printf("%26s", "");
+			printf("AS: %d", p->providers[i].as);
 			switch (p->providers[i].afi) {
 			case AFI_IPV4:
 				printf(" (IPv4 only)");
@@ -685,8 +806,6 @@ takey_print(char *name, const struct takey *t)
 void
 tak_print(const X509 *x, const struct tak *p)
 {
-	char	tbuf[21];
-
 	if (outformats & FORMAT_JSON) {
 		printf("\t\"type\": \"tak\",\n");
 		printf("\t\"ski\": \"%s\",\n", pretty_key_id(p->ski));
@@ -694,16 +813,26 @@ tak_print(const X509 *x, const struct tak *p)
 		printf("\t\"aki\": \"%s\",\n", pretty_key_id(p->aki));
 		printf("\t\"aia\": \"%s\",\n", p->aia);
 		printf("\t\"sia\": \"%s\",\n", p->sia);
-		printf("\t\"valid_until\": %lld,\n", (long long)p->expires);
+		if (p->signtime != 0)
+			printf("\t\"signing_time\": %lld,\n",
+			    (long long)p->signtime);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->notbefore);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->notafter);
+		if (p->expires)
+			printf("\t\"expires\": %lld,\n", (long long)p->expires);
 		printf("\t\"takeys\": [\n");
 	} else {
-		strftime(tbuf, sizeof(tbuf), "%FT%TZ", gmtime(&p->expires));
 		printf("Subject key identifier:   %s\n", pretty_key_id(p->ski));
 		x509_print(x);
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
 		printf("Authority info access:    %s\n", p->aia);
 		printf("Subject info access:      %s\n", p->sia);
-		printf("TAK valid until:          %s\n", tbuf);
+		if (p->signtime != 0)
+			printf("Signing time:             %s\n",
+			    time2str(p->signtime));
+		printf("TAK not before:           %s\n",
+		    time2str(p->notbefore));
+		printf("TAK not after:            %s\n", time2str(p->notafter));
 	}
 
 	takey_print("current", p->current);
@@ -736,14 +865,23 @@ geofeed_print(const X509 *x, const struct geofeed *p)
 		x509_print(x);
 		printf("\t\"aki\": \"%s\",\n", pretty_key_id(p->aki));
 		printf("\t\"aia\": \"%s\",\n", p->aia);
-		printf("\t\"valid_until\": %lld,\n", (long long)p->expires);
+		if (p->signtime != 0)
+			printf("\t\"signing_time\": %lld,\n",
+			    (long long)p->signtime);
+		printf("\t\"valid_since\": %lld,\n", (long long)p->notbefore);
+		printf("\t\"valid_until\": %lld,\n", (long long)p->notafter);
 		printf("\t\"records\": [\n");
 	} else {
 		printf("Subject key identifier:   %s\n", pretty_key_id(p->ski));
 		x509_print(x);
 		printf("Authority key identifier: %s\n", pretty_key_id(p->aki));
 		printf("Authority info access:    %s\n", p->aia);
-		printf("Geofeed valid until:      %s\n", time2str(p->expires));
+		if (p->signtime != 0)
+			printf("Signing time:             %s\n",
+			    time2str(p->signtime));
+		printf("Geofeed not before:       %s\n",
+		    time2str(p->notbefore));
+		printf("Geofeed not after:        %s\n", time2str(p->notafter));
 		printf("Geofeed CSV records:\n");
 	}
 
@@ -756,9 +894,11 @@ geofeed_print(const X509 *x, const struct geofeed *p)
 		if (outformats & FORMAT_JSON)
 			printf("\t\t{ \"prefix\": \"%s\", \"location\": \"%s\""
 			    "}", buf, p->geoips[i].loc);
-		else
-			printf("%5zu: IP: %s (%s)", i + 1, buf,
-			    p->geoips[i].loc);
+		else {
+			if (i > 0)
+				printf("%26s", "");
+			printf("IP: %s (%s)", buf, p->geoips[i].loc);
+		}
 
 		if (outformats & FORMAT_JSON && i + 1 < p->geoipsz)
 			printf(",\n");
