@@ -1,4 +1,4 @@
-/*	$OpenBSD: roa.c,v 1.58 2022/11/29 20:41:32 job Exp $ */
+/*	$OpenBSD: roa.c,v 1.65 2023/03/12 11:54:56 job Exp $ */
 /*
  * Copyright (c) 2022 Theo Buehler <tb@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -211,19 +211,20 @@ roa_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 	struct parse	 p;
 	size_t		 cmsz;
 	unsigned char	*cms;
-	const ASN1_TIME	*at;
 	struct cert	*cert = NULL;
+	time_t		 signtime = 0;
 	int		 rc = 0;
 
 	memset(&p, 0, sizeof(struct parse));
 	p.fn = fn;
 
-	cms = cms_parse_validate(x509, fn, der, len, roa_oid, &cmsz);
+	cms = cms_parse_validate(x509, fn, der, len, roa_oid, &cmsz, &signtime);
 	if (cms == NULL)
 		return NULL;
 
 	if ((p.res = calloc(1, sizeof(struct roa))) == NULL)
 		err(1, NULL);
+	p.res->signtime = signtime;
 
 	if (!x509_get_aia(*x509, fn, &p.res->aia))
 		goto out;
@@ -240,15 +241,10 @@ roa_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 		goto out;
 	}
 
-	at = X509_get0_notAfter(*x509);
-	if (at == NULL) {
-		warnx("%s: X509_get0_notAfter failed", fn);
+	if (!x509_get_notbefore(*x509, fn, &p.res->notbefore))
 		goto out;
-	}
-	if (!x509_get_time(at, &p.res->expires)) {
-		warnx("%s: ASN1_time_parse failed", fn);
+	if (!x509_get_notafter(*x509, fn, &p.res->notafter))
 		goto out;
-	}
 
 	if (!roa_parse_econtent(cms, cmsz, &p))
 		goto out;
@@ -360,8 +356,7 @@ roa_read(struct ibuf *b)
  * number of addresses.
  */
 void
-roa_insert_vrps(struct vrp_tree *tree, struct roa *roa, size_t *vrps,
-    size_t *uniqs)
+roa_insert_vrps(struct vrp_tree *tree, struct roa *roa, struct repo *rp)
 {
 	struct vrp	*v, *found;
 	size_t		 i;
@@ -374,6 +369,10 @@ roa_insert_vrps(struct vrp_tree *tree, struct roa *roa, size_t *vrps,
 		v->maxlength = roa->ips[i].maxlength;
 		v->asid = roa->asid;
 		v->talid = roa->talid;
+		if (rp != NULL)
+			v->repoid = repo_id(rp);
+		else
+			v->repoid = 0;
 		v->expires = roa->expires;
 
 		/*
@@ -387,12 +386,17 @@ roa_insert_vrps(struct vrp_tree *tree, struct roa *roa, size_t *vrps,
 				/* update found with preferred data */
 				found->talid = v->talid;
 				found->expires = v->expires;
+				/* adjust unique count */
+				repo_stat_inc(repo_byid(found->repoid),
+				    RTYPE_ROA, STYPE_DEC_UNIQUE);
+				found->repoid = v->repoid;
+				repo_stat_inc(rp, RTYPE_ROA, STYPE_UNIQUE);
 			}
 			free(v);
 		} else
-			(*uniqs)++;
+			repo_stat_inc(rp, RTYPE_ROA, STYPE_UNIQUE);
 
-		(*vrps)++;
+		repo_stat_inc(rp, RTYPE_ROA, STYPE_TOTAL);
 	}
 }
 
