@@ -1,4 +1,4 @@
-/*	$OpenBSD: validate.c,v 1.51 2022/11/30 08:17:21 job Exp $ */
+/*	$OpenBSD: validate.c,v 1.55 2023/03/06 16:04:52 job Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -396,6 +396,7 @@ valid_x509(char *file, X509_STORE_CTX *store_ctx, X509 *x509, struct auth *a,
 		cryptoerrx("X509_VERIFY_PARAM_add0_policy");
 
 	flags = X509_V_FLAG_CRL_CHECK;
+	flags |= X509_V_FLAG_POLICY_CHECK;
 	flags |= X509_V_FLAG_EXPLICIT_POLICY;
 	flags |= X509_V_FLAG_INHIBIT_MAP;
 	X509_STORE_CTX_set_flags(store_ctx, flags);
@@ -546,3 +547,85 @@ valid_geofeed(const char *fn, struct cert *cert, struct geofeed *g)
 
 	return 1;
 }
+
+/*
+ * Validate whether a given string is a valid UUID.
+ * Returns 1 if valid, 0 otherwise.
+ */
+int
+valid_uuid(const char *s)
+{
+	int n = 0;
+
+	while (1) {
+		switch (n) {
+		case 8:
+		case 13:
+		case 18:
+		case 23:
+			if (s[n] != '-')
+				return 0;
+			break;
+		/* Check UUID is version 4 */
+		case 14:
+			if (s[n] != '4')
+				return 0;
+			break;
+		/* Check UUID variant is 1 */
+		case 19:
+			if (s[n] != '8' && s[n] != '9' && s[n] != 'a' &&
+			    s[n] != 'A' && s[n] != 'b' && s[n] != 'B')
+				return 0;
+			break;
+		case 36:
+			return s[n] == '\0';
+		default:
+			if (!isxdigit((unsigned char)s[n]))
+				return 0;
+			break;
+		}
+		n++;
+	}
+}
+
+int
+valid_ca_pkey(const char *fn, EVP_PKEY *pkey)
+{
+	RSA		*rsa;
+	const BIGNUM	*rsa_e;
+	int		 key_bits;
+
+	if (pkey == NULL) {
+		warnx("%s: failure, pkey is NULL", fn);
+		return 0;
+	}
+
+	if (EVP_PKEY_base_id(pkey) != EVP_PKEY_RSA) {
+		warnx("%s: Expected EVP_PKEY_RSA, got %d", fn,
+		    EVP_PKEY_base_id(pkey));
+		return 0;
+	}
+
+	if ((key_bits = EVP_PKEY_bits(pkey)) != 2048) {
+		warnx("%s: RFC 7935: expected 2048-bit modulus, got %d bits",
+		    fn, key_bits);
+		return 0;
+	}
+
+	if ((rsa = EVP_PKEY_get0_RSA(pkey)) == NULL) {
+		warnx("%s: failed to extract RSA public key", fn);
+		return 0;
+	}
+
+	if ((rsa_e = RSA_get0_e(rsa)) == NULL) {
+		warnx("%s: failed to get RSA exponent", fn);
+		return 0;
+	}
+
+	if (!BN_is_word(rsa_e, 65537)) {
+		warnx("%s: incorrect exponent (e) in RSA public key", fn);
+		return 0;
+	}
+
+	return 1;
+}
